@@ -53,6 +53,7 @@ char *fake = N_("Find the first primitive element in F_q (q must be a prime)");
 char *fake = N_("Find a random primitive element in F_q (q must be a prime)");
 char *fake = N_("Compute discrete log base b of n in F_q (q a prime) using the factor base S.  S should be a column of primes possibly with second column precalculated by IndexCalculusPrecalculation.");
 char *fake = N_("Run the precalculation step of IndexCalculus for logarithms base b in F_q (q a prime) for the factor base S (where S is a column vector of primes).  The logs will be precalculated and returned in the second column.");
+char *fake = N_("Test if Mp is a Mersenne prime using a table");
 char *fake = N_("Tests if a rational number is a perfect power");
 char *fake = N_("Check if g is primitive in F_q, where q is a prime.  If q is not prime results are bogus.");
 char *fake = N_("Check if g is primitive in F_q, where q is a prime and f is a vector of prime factors of q-1.  If q is not prime results are bogus.");
@@ -61,6 +62,7 @@ char *fake = N_("Test if n is a strong pseudoprime to base b but not a prime");
 char *fake = N_("Return the residue of a mod n with the least absolute value (in the interval -n/2 to n/2)");
 char *fake = N_("Test if Mp is a Mersenne prime using the Lucas-Lehmer test");
 char *fake = N_("Return all maximal prime power factors of a number");
+char *fake = N_("Vector with the known Mersenne prime exponents");
 char *fake = N_("Return the Moebius mu function evaluated in n");
 char *fake = N_("Returns the padic valuation (number of trailing zeros in base p).");
 char *fake = N_("Compute a^b mod m");
@@ -160,7 +162,7 @@ char *fake = N_("Get n'th catalan number");
 char *fake = N_("Double factorial: n(n-2)(n-4)...");
 char *fake = N_("Factorial: n(n-1)(n-2)...");
 char *fake = N_("Falling factorial: (n)_k = n(n-1)...(n-(k-1))");
-char *fake = N_("Calculate n'th fibbonachi number");
+char *fake = N_("Calculate n'th Fibonacci number");
 char *fake = N_("Calculate the Frobenius number for a coin problem");
 char *fake = N_("Galois matrix given a linear combining rule (a_1*x_+...+a_n*x_n=x_(n+1))");
 char *fake = N_("Use greedy algorithm to find c, for c . v = n.  (v must be sorted)");
@@ -221,7 +223,6 @@ char *fake = N_("Find root of a function using the method of false position");
 char *fake = N_("Find root of a function using the Muller's method");
 char *fake = N_("Find root of a function using the secant method");
 char *fake = N_("Find roots of a polynomial (given as vector of coefficients)");
-char *fake = N_("Find roots of a quadratic polynomial (given as vector of coefficients)");
 char *fake = N_("Find roots of a quartic polynomial (given as vector of coefficients)");
 char *fake = N_("Use classical non-adaptive Runge-Kutta of fourth order method to numerically solve y'=f(x,y) for initial x0,y0 going to x1 with n increments, returns y at x1");
 char *fake = N_("Calculate average of an entire matrix");
