@@ -1,11 +1,13 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2002 George Lebl
+ * Copyright (C) 1997-2008 Jiri (George) Lebl
  *
- * Author: George Lebl
+ * Author: Jiri (George) Lebl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of Genius.
+ *
+ * Genius is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the  Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- * USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include "config.h"
 
@@ -33,13 +33,14 @@
 /* #define MEM_DEBUG_FRIENDLY 1 */
 
 /*the context stack structure*/
-typedef struct _context_t {
+typedef struct _GelDictContext {
 	GSList *stack;
 	GSList *subststack;
+	GSList *stackname;
 	int top;
-} context_t;
+} GelDictContext;
 
-static context_t context={NULL,NULL,-1};
+static GelDictContext context = {NULL, NULL, NULL, -1};
 
 static GHashTable *dictionary;
 
@@ -313,6 +314,7 @@ d_initcontext(void)
 	/*add an empty dictionary*/
 	context.stack = g_slist_prepend (NULL,NULL);
 	context.subststack = g_slist_prepend (NULL,NULL);
+	context.stackname = g_slist_prepend (NULL,NULL);
 
 	dictionary = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -489,11 +491,39 @@ d_intern (const char *id)
 	return tok;
 }
 
+/* this may be inefficient as it also goes through global,
+ * but we don't assume we do this kind of thing often.  Only 
+ * done in d_delete which is only done on Undefine. */
+static void
+whack_from_all_contexts (GelEFunc *func)
+{
+	GSList *li;
+	for (li = context.stack; li != NULL; li = li->next) {
+		li->data = g_slist_remove (li->data, func);
+	}
+}
+
 gboolean
 d_delete(GelToken *id)
 {
-	/*FIXME: Delete function!*/
-	return FALSE;
+	GSList *li, *list;
+
+	id->protected_ = 0;
+	id->parameter = 0;
+	id->built_in_parameter = 0;
+
+	id->curref = NULL;
+	list = id->refs;
+	id->refs = NULL;
+	for (li = list; li != NULL; li = li->next) {
+		GelEFunc *f = li->data;
+		f->id = NULL;
+		whack_from_all_contexts (f);
+		d_freefunc (f);
+	}
+	g_slist_free (list);
+
+	return TRUE;
 }
 
 /*clear all context dictionaries and pop out all the contexts except
@@ -522,7 +552,7 @@ d_freedict (GSList *n)
 }
 
 static void
-whack_from_lists (GelEFunc *func)
+whack_from_subst_lists (GelEFunc *func)
 {
 	GSList *li;
 	for (li = context.subststack; li != NULL; li = li->next) {
@@ -542,7 +572,7 @@ d_put_on_subst_list (GelEFunc *func)
 		/* On a lower stackframe? */
 		/* weird but true.  So whack it and put it here,
 		 * it will get to the lower one eventually */
-		whack_from_lists (func);
+		whack_from_subst_lists (func);
 	}
 	context.subststack->data = 
 		g_slist_prepend (context.subststack->data, func);
@@ -555,10 +585,12 @@ d_freefunc (GelEFunc *n)
 	GSList *li;
 	if(!n)
 		return;
+	/*
 	g_assert(!n->id || g_slist_find(n->id->refs,n)==NULL);
+	*/
 
 	if (n->on_subst_list) {
-		whack_from_lists (n);
+		whack_from_subst_lists (n);
 		n->on_subst_list = 0;
 	}
 
@@ -592,14 +624,22 @@ void
 d_replacefunc(GelEFunc *old,GelEFunc *_new)
 {
 	GSList *li;
-
-	/* assert some things we don't deal with.  Should we? */
-	g_assert ( ! _new->on_subst_list);
-	g_assert ( ! old->on_subst_list);
+	gboolean put_on_subst = FALSE;
 
 	g_return_if_fail(old && _new);
 	g_return_if_fail(old->id == _new->id);
 
+	if (old->on_subst_list) {
+		whack_from_subst_lists (old);
+		old->on_subst_list = 0;
+	}
+
+	if (_new->on_subst_list) {
+		whack_from_subst_lists (_new);
+		_new->on_subst_list = 0;
+		put_on_subst = TRUE;
+	}
+
 	if(old->type == GEL_USER_FUNC ||
 	   old->type == GEL_VARIABLE_FUNC)
 		gel_freetree(old->data.user);
@@ -614,6 +654,11 @@ d_replacefunc(GelEFunc *old,GelEFunc *_new)
 
 	memcpy(old,_new,sizeof(GelEFunc));
 
+	/* FIXME: this is inefficient */
+	if (put_on_subst) {
+		d_put_on_subst_list (old);
+	}
+
 #ifndef MEM_DEBUG_FRIENDLY
 	/*prepend to free list*/
 	_new->data.next = free_funcs;
@@ -663,6 +708,17 @@ d_addcontext(void)
 {
 	context.stack = g_slist_prepend (context.stack, NULL);
 	context.subststack = g_slist_prepend (context.subststack, NULL);
+	context.stackname = g_slist_prepend (context.stackname, NULL);
+	context.top++;
+	return TRUE;
+}
+
+gboolean
+d_addcontext_named (GelToken *name)
+{
+	context.stack = g_slist_prepend (context.stack, NULL);
+	context.subststack = g_slist_prepend (context.subststack, NULL);
+	context.stackname = g_slist_prepend (context.stackname, name);
 	context.top++;
 	return TRUE;
 }
@@ -705,6 +761,7 @@ d_popcontext(void)
 
 		context.stack = g_slist_delete_link (context.stack, context.stack);
 		context.subststack = g_slist_delete_link (context.subststack, context.subststack);
+		context.stackname = g_slist_delete_link (context.stackname, context.stackname);
 
 		/* substitute lower variables unless we are on the toplevel */
 		if (substlast != NULL && context.top > 0) {
@@ -728,6 +785,24 @@ d_getcontext (void)
 		return context.stack->data;
 }
 
+GSList *
+d_get_all_contexts (void)
+{
+	if (context.top == -1)
+		return NULL;
+	else
+		return context.stack;
+}
+
+GSList *
+d_get_context_names (void)
+{
+	if (context.top == -1)
+		return NULL;
+	else
+		return context.stackname;
+}
+
 /*gimme the current global dictinary*/
 GSList *
 d_getcontext_global (void)
