@@ -1,11 +1,13 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2006 Jiri (George) Lebl
+ * Copyright (C) 1997-2008 Jiri (George) Lebl
  *
  * Author: Jiri (George) Lebl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of Genius.
+ *
+ * Genius is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the  Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- * USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MPWRAP_H_
@@ -151,9 +151,15 @@ mpfr_ptr mpw_peek_imag_mpf (mpw_ptr op);
 
 
 void mpw_abs(mpw_ptr rop,mpw_ptr op);
+void mpw_abs_sq(mpw_ptr rop,mpw_ptr op);
 
 int mpw_sgn(mpw_ptr op);
 
+/* sign of the real part */
+int mpw_re_sgn(mpw_ptr op);
+/* sign of the im part */
+int mpw_im_sgn(mpw_ptr op);
+
 void mpw_neg(mpw_ptr rop,mpw_ptr op);
 
 void mpw_add(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
@@ -163,6 +169,12 @@ void mpw_sub(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
 void mpw_sub_ui(mpw_ptr rop,mpw_ptr op, unsigned long i);
 void mpw_ui_sub(mpw_ptr rop,unsigned long i, mpw_ptr op);
 
+/* exact zero, not a float! */
+gboolean mpw_exact_zero_p (mpw_ptr op);
+
+/* any zero will do */
+gboolean mpw_zero_p (mpw_ptr op);
+
 void mpw_mul(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
 void mpw_mul_ui(mpw_ptr rop,mpw_ptr op, unsigned int i);
 
@@ -248,6 +260,19 @@ char * mpw_getstring (mpw_ptr num,
 		      int integer_output_base,
 		      gboolean add_parenths);
 
+char * mpw_getstring_chop (mpw_ptr num,
+			   int max_digits,
+			   gboolean scientific_notation,
+			   gboolean results_as_floats,
+			   gboolean mixed_fractions,
+			   /* FIXME: solve dependency, this is defined in calc.h */
+			   /* GelOutputStyle */int style,
+			   int integer_output_base,
+			   gboolean add_parenths,
+			   int chop,
+			   int chop_when,
+			   gboolean force_chop);
+
 void mpw_set_str_float(mpw_ptr rop,const char *s,int base);
 
 /*reads only the imaginary part (use add for real part)*/
@@ -285,4 +310,7 @@ void mpw_get_complex_double(mpw_ptr op, double *r, double *i);
 void mpw_denominator(mpw_ptr rop, mpw_ptr op);
 void mpw_numerator(mpw_ptr rop, mpw_ptr op);
 
+/* is the exponent less then -chop_when */
+gboolean mpw_chop_p (mpw_ptr num, int chop_when);
+
 #endif
