@@ -23,25 +23,20 @@
 #define _UTIL_H_
 
 #include <stdlib.h>
-#include <gtk/gtk.h>
-
-/*simple realloc/free functions using g_malloc/g_free confogming to gmp*/
-void * my_malloc(size_t size);
-void * my_realloc(void *ptr,size_t olds,size_t news);
-void my_free(void *ptr,size_t s);
 
 /*shift the sring to the right by n*/
 void shiftstr(char *s,int n);
 
 /*allocate new space in s for p and append it*/
-char * appendstr(char *s,char *p);
+char * appendstr(char *s,const char *p);
 
 /*allocate new space in s for p and prepend it*/
-char * prependstr(char *s,char *p);
+char * prependstr(char *s,const char *p);
 
 /*stack operations using GSList*/
 void stack_push(GSList **stack, gpointer data);
 gpointer stack_pop(GSList **stack);
+gpointer stack_peek(GSList **stack);
 
 /*escape/unescape \n\t\r\b\a\"\\ ... unescape also \<anything>==<anything>,
   nounescape means characters which to leave escaped*/
@@ -49,8 +44,4 @@ char *unescape_string(char *s, char *end, char *nounescape);
 /*escape also "'s and \'s*/
 char *escape_string(char *s);
 
-/*get the terminal width, 80 is returned if we fail somehow*/
-/*this function is not defined in util.c but in the appropriate main file!!!*/
-/*int get_term_width(void);*/
-
 #endif
